"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parsePropValues;

var _reactDocgen = require("react-docgen");

/**
 * Given a relative path and the React component source, returns an object of all
 * enumerated and boolean prop names mapped to arrays of their possible values.
 *
 * #### Example
 *
 * Given the following component:
 *
 * ```js
 * class MyComponent extends React.Component {
 *   static propTypes = {
 *     shape: PropTypes.oneOf(['square', 'rectangle', 'circle']),
 *     display: PropTypes.bool,
 *     children: PropTypes.node
 *   }
 * }
 * ```
 *
 * The prop values could be parsed via the `parsePropValues` function as follows:
 *
 * ```js
 * import parsePropValues from '@temporg/ui-component-examples'
 *
 * const sourceFilePath = 'path/to/MyComponent'
 * const propValues = parsePropValues(fs.readFileSync(sourceFilePath, 'utf8'))
 * ```
 *
 * The result of the `parsePermutations` call above would be the following object
 *
 * ```js
 * {
 *   shape: ['square', 'rectangle', 'circle'],
 *   display: [true, false]
 * }
 * ```
 */
function parsePropValues(fileSource) {
  var parsedSrc = {};

  try {
    parsedSrc = (0, _reactDocgen.parse)(fileSource, _reactDocgen.resolver.findAllExportedComponentDefinitions);

    if (Array.isArray(parsedSrc)) {
      parsedSrc = parsedSrc.pop();
    }
  } catch (error) {
    throw new Error("[ui-component-examples] Could not parse component source: ".concat(error));
  }

  return getPropValuesFromParsedProps(parsedSrc.props);
}

function getPropValuesFromParsedProps(parsedProps) {
  if (!parsedProps) return [];
  var propValues = {};
  Object.keys(parsedProps).forEach(function (propName) {
    var prop = parsedProps[propName];

    if (!(prop && prop.type)) {
      return;
    }

    var _prop$type = prop.type,
        name = _prop$type.name,
        value = _prop$type.value;

    if (name === 'enum') {
      propValues[propName] = Array.isArray(value) ? value.map(function (entry) {
        var result = entry.value.replace(/['"]+/g, '');
        return result === 'null' ? null : result;
      }) : [];
    } else if (name === 'bool') {
      propValues[propName] = [false, true];
    }
  });
  return propValues;
}